c-------------------------------------------------------------------------------
c Subroutine used to evaluate the potential inside the medium
c
c Yijun Liu											      October 12, 2004
c-------------------------------------------------------------------------------

      subroutine domain_field(nfield,xfield,f,n,x,y,bc,node,dnorm,u)

      implicit real*8(a-h,o-z)

      dimension xfield(2,*), f(*), x(2,*),y(2,*),bc(2,*),node(2,*),
     &          dnorm(2,*),u(*)

      data      pi/3.141592653589793D0/

	pi2 = pi*2.d0

	do i=1,nfield
        f(i) = 0.d0
	enddo

      do j=1,n      ! Loop over all elements

	  if(bc(1,j).eq.1) then
	    f0  = bc(2,j)
	    df0 = u(j)
	  else if(bc(1,j).eq.2) then
	    f0  = u(j)
	    df0 = bc(2,j)
	  endif

	  al   = sqrt((y(1,node(2,j))-y(1,node(1,j)))**2 +
     &              (y(2,node(2,j))-y(2,node(1,j)))**2)         ! Element length

	do i=1,nfield     ! Loop over all field points inside the domain

        x11 = y(1,node(1,j)) - xfield(1,i)
        x21 = y(2,node(1,j)) - xfield(2,i)
        x12 = y(1,node(2,j)) - xfield(1,i)
        x22 = y(2,node(2,j)) - xfield(2,i)

        r1 =  sqrt(x11**2 + x21**2)
        r2 =  sqrt(x12**2 + x22**2)
        d  =  x11*dnorm(1,j) + x21*dnorm(2,j)
        t1 = -x11*dnorm(2,j) + x21*dnorm(1,j)
        t2 = -x12*dnorm(2,j) + x22*dnorm(1,j)

	  ds = abs(d)
	  theta1 = datan2(t1,ds)
	  theta2 = datan2(t2,ds)
	  dtheta = theta2 - theta1

        aa = (-dtheta*ds + al + t1*log(r1)-t2*log(r2))/pi2
	  if(d .lt. 0.d0) dtheta = -dtheta
	  bb = -dtheta/pi2

	  f(i) = f(i) + aa*df0 - bb*f0

	enddo
	enddo

c Output results

	do i=1,nfield
	  write(9,20) xfield(1,i), f(i)
	enddo
20    format(1x, 4E18.8)

      return
      end